#"G2F 2014-2016 Individual Site Analysis to get Hybrid BLUEs"
#Updated to restrict residual spatial correlations to -0.6 < r < +0.6
#This limit recommended by Arthur Gilmour based on experience with replicated trials
#In trials with limited replication we cannot estimate the correlations very well and allowing them to go too high causes weights to get too high
#And truncate hybrid names to 60 characters to they will be matched properly to relationship matrix in Stage 2 analysis
#This only affects one hybrid: (CML442-B/CML343-B-B-B-B-B-B)-B-B-1-1-B-B-B-1-B12-1-B19/LH195 - the final character '5' gets dropped
#And specify prediction at center of linear and quadratic stand covariates

library(tidyverse)
library(xml2)
library(asreml)
in_path = "Q:/My Drive/Anna/asreml_3yrs/Stage 1 BLUEs/stage1_out/"
out_path = "Q:/My Drive/Anna/asreml_3yrs/Stage 1 BLUEs/"
asremlpath <- '"C:/Program Files/ASReml4/bin/asreml"' 
setwd(out_path)

#SET THE LIMIT ON ABSOLUTE VALUE OF SPATIAL CORRELATIONS
max.ar1 = 0.6

#get cleaned plot level trait data
g2f = read.csv('Q:/My Drive/Anna/G2F Phenotype Data/ThreeYearPheno_65Envfiltered_10302018.csv')

#make sure Rep, Block, and Hybrid are factors
#Range, Pass should be int for the augmented grid code to work properly
g2f = mutate(g2f, 
       Rep = as.factor(Rep),
       Block = as.factor(Block),
       Hybrid = as.factor(Hybrid))

#drop environments that were inoculated with NLB:
inoc.envs = c('NYH2_2014', 'NYH1_2015', 'NYH1_2016') 

g2f = filter(g2f, ! Env %in% inoc.envs)

#Write an .as script to fit the diallel model in asreml ... can't do it in asreml-R since we need the and() function!
#Have to execute it in standalone asreml.
#!NOGRAPHS was not being recognized, so I used !HARDCOPY option to suppress interactive graphics display
script = 
paste(
  "!HARDCOPY !EPS !XML !WORKSPACE 8 !RENAME !OUTFOLDER stage1_out 
Title: STAGE 1 of TWO-STAGE ANALYSIS OF G2F 2014-16 DATA 
Year  !I  3 !SORTALL   # 2014
Experiment  !A  37    # DEH1
!CSKIP 1
#RecId  !I      # 2209152
State  !A   40   # DE
City  !A    65  # Georgetown
Hybrid_Code  !I  2422    # 736
!CSKIP 1
#Source  !A      # 13WJWE:LH185:2061
Hybrid  !A 2422 !LL 61 !SPECIALCHAR    # M0346/LH185
!CSKIP 1
#Exp_Code  !A   100   # 25#looks like this column is wrong in 2016
Heterotic_Pool  *       # 1
Rep  *       # 1
Block  *       # 1
Plot  !I  1436    # 29
Range  *       # 3
Pass  *       # 9
!CSKIP 5
#Plot_area        # 77.1
#Date_Planted  !I      # 5/5/2014
#Date_Harvested  !I      # 9/29/2014
#Anthesis_date  !I      # 7/7/2014
#Silking_date  !I      # 7/9/2014
Stand_Count_plants       # 55
Pollen_DAP       # 63
Silk_DAP        # 65
Plant_Height_cm
Ear_Height_cm         # 0
Root_Lodging_plants         # 1
Stalk_Lodging_plants        # 0.713359273670558
Grain_Moisture        # 18.7
Test_Weight_lbs_bu        # 53.9
Plot_Weight_lbs        # 19.61
Yield   #!D qualifier discard any records with missing yield values    # 190.3517598
!CSKIP 3
#Plot_Discarded  !A      # NA
#Filler    !A    # NA
#Comments  !A      # NA
Stand_per_area        # -0.004945053392448
Stand_per_area_mn
stand_lin_cov        # 2.44535530541614e-05
stand_quad_cov        # 0.718304327063006
Env !A !SORT 72 #environment code = Experiment_Year
Yield_Mg_ha	
Twt_kg_m3
Stand_per_ha
Lodging 

'Q:/My Drive/Anna/asreml_3yrs/Stage 1 BLUEs/G2F_local_subset.csv'  !SKIP 1 !MAXITER 100 !DDF 1  !DOPART 1 !DENSE 2427 !FCON  !SECTION Experiment !Rowfac Range !Colfac Pass

!PART 1##basic anova model at 1 env only
!arlim ", max.ar1, "

ZZZFIXED ,         
!r   ZZZRANDOM 
residual ZZZRESID
predict Hybrid !PRESENT Hybrid !TWOSTAGEWEIGHTS
")

#save the code to the job command file for asreml
#already did this, so can skip for now
#sink(file = 'G2F2014-16stage1_.as')
#cat(script)
#sink()

#Make a windows batch file to run this script  
#I already made this file, called asreml_batch.cmd  
#it is a plain text file with these contents:  

# batfile = '
# set MyPath=C:\\Program Files\\ASReml4\\bin  
# "%MyPath%\\ASReml.exe" G2F2014-16stage1.as
# '
# 
# sink(file = 'Q:/My Drive/Anna/asreml_3yrs/Stage 1 BLUEs/asreml_batch.cmd')
# cat(batfile)
# sink()

#for each environment and trait, test a series of models and select best fitting model for use later
covars = '+ stand_lin_cov + stand_quad_cov'

#make empty lists to hold results for each environment - trait combination
results.list = list()
vc.list = list()
pred.list = list() #empty list to hold prediction results
F.list = list()
cov.list = list()
error.list = list() #list to hold information on environment-trait combinations that had convergence problems
skip.list = list() #list to hold information on environment-trait combinations that were skipped due to limited data

env.list = sort(unique(g2f$Env))
trait.list = c('Yield_Mg_ha', 'Grain_Moisture', 'Twt_kg_m3', 'Lodging', 'Pollen_DAP', 'Silk_DAP', 'Plant_Height_cm', 'Ear_Height_cm')

for (env in env.list){
    for (trait in trait.list){
    df = g2f %>% filter(Env == env) 
    df = df[! is.na(df[[trait]]),] 
    #do some QC checks 
    if (nrow(df) < 30) {skip.list[[paste0(env, "_", trait)]] = "<30 observations"; next} #if total observations < 30, skip
    if (var(df[[trait]]) == 0) {skip.list[[paste0(env, "_", trait)]] = "No trait variation"; next} #if no variance in trait, skip
    #check if at least 20 df are available to estimate residual variance (ignoring possibility of fitting covariates and blocks)
    dfe = df %>% group_by(Hybrid) %>% summarise(N = n()) %>% filter(N > 1) %>% mutate(dfe = N - 1) %>% summarise(sum(dfe))
    if (dfe < 20) {skip.list[[paste0(env, "_", trait)]] = "df.error < 20"; next}
    
    #select if blocks or reps are part of design at this environment
    
    if (length(unique(df$Rep)) == 1) {random = NA} else {
      if ((length(unique(df$Block)) <= length(unique(df$Rep))) | (dfe < 40)) {random = '~ Rep '} else {random = '~ Rep + Block:Rep'} # if dfe < 40 df, do not include block effects
    } 
    
    #for yield, compare models with and without stand covariates and random blocking structures
    #select best model
    #the stand covariates are orthogonal, so we can just fit one model with both at the same time, 
    #then check their p-values
    if (trait == "Yield_Mg_ha") {
      fixed = paste(trait, '~ Hybrid', covars)
      if (is.na(random)){
        mod = asreml(fixed = as.formula(fixed), data = df, maxiter = 25)
      } else {
        mod = asreml(fixed = as.formula(fixed), as.formula(random), data = df, maxiter = 25)
      }
      
    #check if either or both covariates are significant at p = 0.05
    Ftests = wald(mod)
    fixed.sig = rownames(Ftests[Ftests[,'Pr(Chisq)'] < 0.05,])
    covars.sig = fixed.sig[fixed.sig %in% c('stand_lin_cov', 'stand_quad_cov')]
    if (length(covars.sig) == 0) covars.sig = '' else covars.sig = paste('+', covars.sig, collapse = ' ')
    #now covars.sig consists only of the significant covariate(s)
    #update fixed
    fixed = paste(trait, '~ Hybrid', covars.sig)
  }  else  fixed = paste(trait, '~ Hybrid') #end of covariate selection, or set fixed effect model for non-yield traits
  
####Current best model is the base model with units variance  
if (is.na(random)){
      best.mod = asreml(fixed = as.formula(fixed), data = df, maxiter = 25)
    } else {
      best.mod = asreml(fixed = as.formula(fixed), random = as.formula(random), data = df, maxiter = 25)
    }
#BIC = ln(n)*k - 2*logL, asreml manual uses n = number of residual df, not total observations
best.bic = (log(best.mod$nedf)*(length(best.mod$gammas))) - (2*best.mod$loglik)
best.r = 'units'
nugget = F
local.exps = as.character(unique(df$Experiment))
num.exps = length(local.exps)

####FIND THE BEST RESIDUAL VARIANCE STRUCTURE IF THERE IS RANGE AND PASS INFORMATION  
   if (!(any(is.na(df$Range)) | any(is.na(df$Pass)))) {
     rstructs = c('ar1v(Range):ar1(Pass)', 'idv(Range):ar1(Pass)', 'ar1(Range):idv(Pass)')
     
    #for asreml-R need to augment the data to make a complete grid of range and pass
    #this is complicated for environments with more than one experiment combined,
    #since each experiment is an independent 'section'
     ex.list = list()
     for (ex in 1:num.exps) {
        ex.df = df[df$Experiment == local.exps[ex], ]
        max.range = max(as.numeric(ex.df$Range), na.rm = T)
        max.pass = max(as.numeric(ex.df$Pass), na.rm = T)
        aug.df = expand.grid(Experiment = local.exps[ex], Range = 1:max.range, Pass = 1:max.pass)
        ex.df = merge(ex.df, aug.df, by = c("Experiment", "Range", "Pass"), all = T)
        ex.list[[ex]] = ex.df
      }
       
     df2 = do.call(rbind, ex.list) 
     
     if (num.exps > 1) rstructs = paste0("at(Experiment):", rstructs)
     
     #Need to fill in missing values for factors that are fit in model, 
     #reset model factors to factors
     df2 = mutate(df2, 
                 Experiment = factor(Experiment),
                 Rep = factor(ifelse(is.na(Rep),1, Rep)), 
                 Block = factor(ifelse(is.na(Block),1, Block)),
                 Range = factor(Range),
                 Pass = factor(Pass),
                 stand_lin_cov = ifelse(is.na(stand_lin_cov),0, stand_lin_cov),
                 stand_quad_cov = ifelse(is.na(stand_quad_cov),0, stand_quad_cov),
                 Hybrid = ifelse(is.na(Hybrid),"MISSING", Hybrid))
 
    for (r in rstructs){
      name = paste(env, trait, fixed, random, r)
      print(name)
      if (is.na(random)){
        mod2 = asreml(fixed = as.formula(fixed), rcov = as.formula(paste('~', r)), data = df2, maxiter = 30)
        } else {
        mod2 = asreml(fixed = as.formula(fixed), random = as.formula(random), rcov = as.formula(paste('~', r)), data = df2, maxiter = 30)
        }
      curr.bic = (log(best.mod$nedf)*(length(mod2$gammas))) - (2*mod2$loglik)
      if (curr.bic < best.bic){
        best.mod = mod2
        best.bic = curr.bic
        best.r = r
      }
   }

    #if best model includes a spatial correlation, then also try adding nugget variance to that
    if (best.r != "units") {
      "Add units to best spatial model"
      if (is.na(random)) newrand = "random = ~ units" else newrand = paste(random, "+ units")
      units.mod = asreml(fixed = as.formula(fixed), as.formula(newrand), rcov = as.formula(paste('~', best.r)), data = df2, maxiter = 30)
      units.bic = (log(best.mod$nedf)*(length(units.mod$gammas))) - (2*units.mod$loglik)
      #best.mod is updated to units model if it has lower BIC than current best.mod
      if (units.bic < best.bic) {
        best.mod = units.mod
        best.bic = units.bic
        nugget = T
        random = newrand
        }
   } #end of nugget test
  } else { ####END OF FINDING BEST RESIDUAL STRUCTURE 
    df2 = df} ###IF NO SPATIAL ANALYSIS, CREATE df2 FOR USE BELOW

#For the best model, write a version with random hybrids and estimate heritability using Cullis method
if (is.na(random)) random.term = "~ Hybrid" else random.term = paste(random, '+ Hybrid')

#for some reason, update() fails on some models where it should not
#rand.mod = update(best.mod, fixed. = . ~ . - Hybrid, random = as.formula(random.term))

#get initial values from best model
#and at the same time:#final step for spatial models is to check if correlations are above 0.5 (or below -0.5)
#as this usually is a sign of unstable parameter values and results in highly inflated weights
#we will restrict spatial correlations to 0.6 or less to ensure that weights are stable
rand.mod = asreml(fixed = as.formula(sub("Hybrid", "1", fixed, fixed = T)),
                  random = as.formula(random.term),
                  rcov = as.formula(paste("~", best.r)), 
                  data = df2, 
                  maxiter = 30)

cor.components = grep(".cor$", names(rand.mod$gammas), value = T)
if (length(cor.components > 0)) {
  check.cor = any(abs(rand.mod$gammas[names(rand.mod$gammas) %in% cor.components]) > max.ar1)} else check.cor = FALSE

if (check.cor){
  rand.mod.sv = update(rand.mod, start.values = T)$gammas.table
  rand.mod.sv[rand.mod.sv$Gamma %in% cor.components & abs(rand.mod.sv$Value) > max.ar1, 'Constraint'] = 'F'
  rand.mod.sv[rand.mod.sv$Gamma %in% cor.components & rand.mod.sv$Value > max.ar1, 'Value'] = max.ar1
  rand.mod.sv[rand.mod.sv$Gamma %in% cor.components & rand.mod.sv$Value < -max.ar1, 'Value'] = -max.ar1
  
  rand.mod = asreml(fixed = as.formula(sub("Hybrid", "1", fixed, fixed = T)),
                    random = as.formula(random.term),
                    rcov = as.formula(paste("~", best.r)), 
                    data = df2, 
                    maxiter = 30,
                    R.param = rand.mod.sv)
}

rand.mod.pred = predict(rand.mod, classify = "Hybrid", present = "Hybrid")
avSED = rand.mod.pred$predictions$avsed
Vg = summary(rand.mod)$varcomp["Hybrid!Hybrid.var", "component"]
H.mean = 1 - ((avSED**2)/(2*Vg)) #Heritability of hybrid means
#Plot basis heritability not quite right for spatial models, but compute it anyway
#Verr is ALSO incorrect here for spatial models because "variance" term gets fixed at 1, and the variance components in Range and Pass directions
#should be used. I left this computation in here, but Anna is fixing it in another script post-hoc using the outputted variance components.
Verr = sum(summary(rand.mod)$varcomp[grep("variance", rownames(summary(rand.mod)$varcomp)),"component"])
if (is.na(Verr)) Verr = 0
if (nugget) Verr = Verr + summary(rand.mod)$varcomp["units!units.var", "component"]
H.plot = Vg/(Vg + Verr)

#Save the variance components, etc. into a data frame, then add to results list
results.df = data.frame(
  Env = env,
  Trait = trait,
  Fixed = fixed,
  Random = random,
  R.structure = best.r,
  Verr = Verr,
  H.plot = H.plot,
  H.mean = H.mean)

  vcomps.df = summary(rand.mod)$varcomp %>%
  mutate(Env = env,
         Trait = trait,
         Factor = rownames(.))

  results.list[[paste(env, trait)]] = results.df
  vc.list[[paste(env, trait)]] = vcomps.df

  #modify the script
  fixed.standalone = gsub("~", "~ mu + ", fixed)
  fixed.standalone = paste(fixed,  "!f mv")
  random.standalone = gsub("~", "", random)
  script.new = gsub("ZZZFIXED", fixed.standalone, script)
  script.new = gsub("ZZZRANDOM", random.standalone, script.new)
  script.new = gsub("ZZZRESID", best.r, script.new)
  
  #if we have included covariate(s) in the model, need to include them in the prediction statement
  if (grepl('stand_lin_cov', fixed) & grepl('stand_quad_cov', fixed)){
    script.new = sub("predict Hybrid !PRESENT Hybrid !TWOSTAGEWEIGHTS", "predict Hybrid stand_lin_cov 0. stand_quad_cov 0. !PRESENT Hybrid !TWOSTAGEWEIGHTS", script.new)}
  if (grepl('stand_lin_cov', fixed) & !grepl('stand_quad_cov', fixed)){
    script.new = sub("predict Hybrid !PRESENT Hybrid !TWOSTAGEWEIGHTS", "predict Hybrid stand_lin_cov 0. !PRESENT Hybrid !TWOSTAGEWEIGHTS", script.new)}
  if (!grepl('stand_lin_cov', fixed) & grepl('stand_quad_cov', fixed)){
    script.new = sub("predict Hybrid !PRESENT Hybrid !TWOSTAGEWEIGHTS", "predict Hybrid stand_quad_cov 0. !PRESENT Hybrid !TWOSTAGEWEIGHTS", script.new)}
  
  #if best model has a spatial correlation and multiple experiments at the environment,
  #then asreml standalone has a problem fitting the model
  #need to use only a subset of data for the current environment and add some special qualifiers
  #this problem also rarely occurs even with units residual, so use this option always
  write.csv(df, "Q:/My Drive/Anna/asreml_3yrs/Stage 1 BLUEs/G2F_local_subset.csv", row.names = F, quote = F)

  #in cases where more than one experiment, but units residual, we need to remove the !SECTION experiment qualifier
  if (best.r == "units" & num.exps > 1) script.new = sub("!SECTION Experiment", "", script.new)
  
    #write the script to hard drive
    sink(file = 'Q:/My Drive/Anna/asreml_3yrs/Stage 1 BLUEs/G2F2014-16stage1.as')
    cat(script.new)
    sink()
    
    #run the script
    shell('asreml_batch.cmd')
    #shell(paste0('\"',out_path,'asreml_batch.cmd','\"'))
    
    #catch the output from the xml file and parse it
    fileName = paste0(out_path, 'stage1_out/G2F2014-16stage1.xml')
    raw_xml <- readChar(fileName, file.info(fileName)$size)

    outp = read_xml(fileName) 
    
    #check for convergence, skip results if not
    converg = trimws(xml_text(xml_find_all(outp, '//ASReport/Cycle/Conclusion')))
    if (!grepl("LogL Converged", converg)) { #CATCH ENVIRONMENTS WITH CONVERGENCE PROBLEMS
      error.list[paste(env, trait)] = converg
      next}
  
  #Get F-tests and p-values of fixed effects
  AOV_list = as_list(xml_find_all(outp, '//ASReport/Cycle/WaldFstats/WaldFtest'))
  #This is a list of lists, need to step through it, compiling a data frame
  AOV.list = list()
  for (i in 1:length(AOV_list)) AOV.list[[i]] = as.data.frame((do.call(cbind, AOV_list[[i]])))
  F.df = do.call(rbind, AOV.list)
  colnames(F.df) = gsub("-", "_", colnames(F.df)) 
  
  #in cases where we have multiple experiments at an environment with spatial models
  #Asreml fails to compute DenDF and pvalues for fixed effects, so we provide an alternative approximation
  #it's a poor estimator, but once DDEF is large, it has little influence on p-vals, so we use it
  if (! "DenDF" %in% colnames(F.df)) F.df = mutate(F.df, DenDF = best.mod$nedf, Probability = NA)

  F.df = F.df %>%
    mutate(ModelTerm = trimws(ModelTerm),
           NumDF = as.numeric(NumDF),
           DenDF = as.numeric(DenDF),
           F_con = as.numeric(F_con),
           Probability = gsub( "<", "", Probability),
           Probability = as.numeric(Probability), 
           Probability = ifelse(is.na(Probability), pf(F_con, NumDF, DenDF, lower.tail = F), Probability),
           Env = env, 
           Trait = trait) %>%
    select(-MarginFlag, -F_inc, -FactorNumber)
  
   F.list[[paste(env, trait, sep = "_")]] = F.df
  
  #get the covariate effect estimates, IF covariate was used
  if (grepl('stand_', fixed, fixed = T)){
   path.cov = xml_find_all(outp, "//ASReport/Cycle/Solutions/Equation")
   indx = trimws(xml_text(xml_find_first(path.cov, ".//Model_Term"))) %in% c("stand_quad_cov", "stand_lin_cov")
   cov.xml = as_list(path.cov[indx])
   c.list = list()
   for (i in 1:length(cov.xml)) c.list[[i]] = as.data.frame((do.call(cbind, cov.xml[[i]])))
   cov.df = do.call(rbind, c.list) %>%
     mutate(Env = env, 
            Trait = trait)
   cov.list[[paste(env, trait, sep = "_")]] = cov.df}
 
  #get the predicted mean values for Hybrids and weights for 2nd stage
  path.pred = "//ASReport/Cycle/PredictTable/Prow"
  pred.cell = as.numeric(xml_text(xml_find_all(outp, paste0(path.pred, "/Cell"))))
  hybrid.name = xml_text(xml_find_all(outp, paste0(path.pred, "/Identifier")))
  pred.value = as.numeric(xml_text(xml_find_all(outp, paste0(path.pred, "/PredValue"))))
  #get the prediction variance and effective replication values to use as weights
  pred.var = as.numeric(xml_text(xml_find_all(outp, paste0(path.pred, "/Variance"))))
  #path.2stage = "//ASReport/Cycle/PredictTable/TwoStageWeight"
  pred.er.cell = as.numeric(xml_text(xml_find_all(outp, paste0(path.pred, "/TwoStageWeight/Cell"))))
  pred.er = as.numeric(xml_text(xml_find_all(outp, paste0(path.pred, "/TwoStageWeight/Weight"))))

  #combine all the prediction info together into a single data frame and compute weights for 2nd stage
  preds1 = data.frame(Env = env, Trait = trait, Cell = pred.cell, Hybrid = hybrid.name, BLUE = pred.value)
  preds2 = data.frame(Cell = pred.er.cell, Eff_Repl = pred.er)
  preds = merge(preds1, preds2)
  preds = mutate(preds, weight = Eff_Repl/pred.var)

  pred.list[[paste(env, trait, sep = "_")]] = preds
  
  rm(fixed, random, best.mod, best.bic, best.r)
  } #end trait loop
} #end env loop

#compile results into data frames
vc.df = do.call(rbind, vc.list)

#######REST OF SCRIPT IS QC CHECKS ON RESULTS AND PREDICTIONS
#######AND WRITING RESULT FILES

#check we have the expected results
print("Environments included in results")
unique(vc.df$Env)
print("Traits included in results")
unique(vc.df$Trait)

#did any environments fail to converge?
print("Bad environments - no convergence")
print(error.list) #These environment results were not saved in summary stats or prediction objects.

#combine results into a data frame
results.df = do.call(rbind, results.list)
results.df = mutate(results.df, Env = as.character(Env))

#identify and remove environment-trait combinations with mean heritability < 0.1
#also remove all traits from environments with yield heritability < 0.1 (but actually this did not happen in 2014-16!)
bad.envs = results.df %>% filter(H.mean < 0.1) 
bad.envs = bad.envs %>%
  mutate(Env_Trait = paste(Env, Trait, sep = '_'))

bad.envs.yield = bad.envs %>% filter(Trait == "Yield_Mg_ha") %>% select(Env) %>% unlist() #this is a vector
bad.envs.other = bad.envs %>% filter(!Env %in% bad.envs.yield) %>% select(Env, Trait) #this is a data frame

results.df = results.df %>% filter(H.mean >= 0.1)
print("Summary of environment results after filtering for low heritability")
summary(results.df)

#check for env-trait combination(s) with zero error variance?
print("Environments with very low error variance")
results.df %>% filter(Verr < 0.01)
#these look OK, they are all associated with lodging and have reasonable heritability, so it is a scale issue

#check for env-trait combinations with near 1.0 heritability?
print("Environments with very high mean heritability")
results.df %>% filter(H.mean > 0.95)
#look ok

write.csv(results.df, file = paste0(out_path, "G2F_2014-16_Stage1_ModelInfo.csv"))

#compile results into data frames
vc.df = do.call(rbind, vc.list)
#check we have the expected results
unique(vc.df$Env)
unique(vc.df$Trait)

#remove bad env-trait combinations #another slick way to do it is:
#preds.filtered = anti_join(preds.filtered, bad.envs.other)
vc.df = vc.df %>% 
  mutate(Env_Trait = paste(Env, Trait, sep = '_')) %>%
  filter(! Env_Trait %in% bad.envs$Env_Trait)

#check the variation in residual variance components among environments
print("Summary of Yield error variance among environments")
vc.df %>% filter(grepl("variance", Factor) & Trait == "Yield_Mg_ha") %>% select(component) %>% summary()

print("Environments with low or high Yield error variance")
vc.df %>% filter(grepl("variance", Factor) & Trait == "Yield_Mg_ha",
                 component < 1 | component > 3)

#write out the variance components results by environment and trait
write.csv(vc.df, file = paste0(out_path, "G2F_2014-16_Stage1_VarCompEsts.csv"), row.names = F, quote = F)

vc.wide = select(vc.df, Env, Trait, Factor, component, -Env_Trait) %>%
  spread(key = Trait, value = component) %>%
  arrange(Env)
write.csv(vc.wide, file = paste0(out_path, "G2F_2014-16_Stage1_VarCompEsts_wide.csv"), row.names = F, quote = F)

#combine F-tests for fixed effects and write to file
F.df.final = do.call(rbind, F.list)
F.df.final = F.df.final %>% 
  mutate(Env_Trait = paste(Env, Trait, sep = '_')) %>%
  filter(! Env_Trait %in% bad.envs$Env_Trait)
write.csv(F.df.final, file = paste0(out_path, "G2F_2014-16_Stage1_Fvalues.csv"), row.names = F, quote = F)

#get the stand covariate effect estimates for each environment for yield
#The elements of cov.df are still lists, need to unlist them to vectors
cov.df2 = cov.df %>% mutate(Model_Term = trimws(unlist(Model_Term, use.names = F)), 
                            Effect = as.numeric(unlist(Effect, use.names = F)),
                            seEffect = as.numeric(unlist(seEffect, use.names = F)),
                            Env_Trait = paste(Env, Trait, sep = '_')) %>%
  filter(! Env_Trait %in% bad.envs$Env_Trait) %>%
  select(-Level)
write.csv(cov.df2, file = paste0(out_path, "G2F_2014-16_Stage1_covariate_effects.csv"), row.names = F, quote = F)

#combine the predictions (BLUEs)
pred.df = do.call(rbind, pred.list) %>% select(-Cell) %>%
  #trim white space from Hybrid Names and get rid of 'Cell' column
  #AND TRUNCATE TO 60 CHARACTERS MAX!!!
  mutate(Hybrid = substr(trimws(as.character(Hybrid)),1,60),
         Env_Trait = paste(Env, Trait, sep = '_')) %>%
  filter(! Env_Trait %in% bad.envs$Env_Trait)

pred.wide = pred.df %>%
  select(-Env_Trait, -Eff_Repl) %>%
  gather(key = Param, value = "value", BLUE, weight) %>%
  unite(Trait_param, Trait, Param) %>%
  spread(key = Trait_param, value = value)

#QC check if any hybrids in pred.df missing from pred.wide 
print("Any hybrids in pred.df but missing in pred.wide?")
pred.wide %>% filter(! Hybrid %in% unique(pred.df$Hybrid))

#QC check if any environments are missing from prediction file but are in the vc.df
print("Any environments in vc.df but missing from prediction file?")
vc.df %>% filter(! Env %in% unique(pred.df$Env))

#QC for BLUEs
for (trait in c( "Yield_Mg_ha_BLUE" ,  "Twt_kg_m3_BLUE" ,  "Silk_DAP_BLUE", "Plant_Height_cm_BLUE" , "Ear_Height_cm_BLUE", "Grain_Moisture_BLUE")) {
  print(trait)
  x = pred.wide[[trait]]
  print(summary(x))
  sd.x = sd(x, na.rm = T)
  iq.x = IQR(x, na.rm = T)
  mean.x = mean(x, na.rm = T)
  top.thresh = mean.x + (2*iq.x)
  bot.thresh = mean.x - (2*iq.x)
  if (trait == "Twt_kg_m3_BLUE"){
    top.thresh = mean.x + (2.5*iq.x)
    bot.thresh = mean.x - (2.5*iq.x)
  }
  hist(x, main = paste("Histogram of Env-specific BLUEs for", trait))
  abline(v = c(bot.thresh, top.thresh), col = 'red')
  print("Number of observations beyond thresholds")
  print(nrow(pred.wide[(x < bot.thresh | x > top.thresh) & !is.na(x),]))
}

#Check scatterplot of DTA and DTS to find outliers
plot(pred.wide$Pollen_DAP_BLUE, pred.wide$Silk_DAP_BLUE, main = "DTA vs DTS BLUEs")

#For yield, drop outside of 2*IQR, for twt 3.5*IQR

#check height outliers using scatterplot, initial inspection of high values shows that some are
#BGEM topcrosses, which are expected to be very tall, so be careful about removing these
hi.ht = pred.wide %>% filter(Ear_Height_cm_BLUE > 160)
ht.outliers = pred.wide %>% filter(Ear_Height_cm_BLUE < 60 | Ear_Height_cm_BLUE > 160 | Plant_Height_cm_BLUE < 150 | Plant_Height_cm_BLUE > 300)

plot(ht.outliers$Ear_Height_cm_BLUE, ht.outliers$Plant_Height_cm_BLUE)
too.tall = ht.outliers %>% filter(Plant_Height_cm_BLUE > 300 & Ear_Height_cm_BLUE < 100)
too.short = ht.outliers %>% filter(Plant_Height_cm_BLUE <180 & Ear_Height_cm_BLUE > 150)
points(too.tall$Ear_Height_cm_BLUE, too.tall$Plant_Height_cm_BLUE, col = "red")
points(too.short$Ear_Height_cm_BLUE, too.short$Plant_Height_cm_BLUE, col = "blue")

#let's remove the outliers, setting them to NA
iq.yield = IQR(pred.wide$Yield_Mg_ha_BLUE, na.rm = T)
mean.yield = mean(pred.wide$Yield_Mg_ha_BLUE, na.rm = T)
iq.twt = IQR(pred.wide$Twt_kg_m3_BLUE, na.rm = T)
mean.twt = mean(pred.wide$Twt_kg_m3_BLUE, na.rm = T)

pred.qc = pred.wide %>%
 mutate(Yield_Mg_ha_BLUE = ifelse(Yield_Mg_ha_BLUE < (mean.yield + 2*iq.yield), Yield_Mg_ha_BLUE , NA),
         Yield_Mg_ha_BLUE = ifelse(Yield_Mg_ha_BLUE > (mean.yield - 2*iq.yield), Yield_Mg_ha_BLUE , NA),
         Yield_Mg_ha_BLUE = ifelse(Yield_Mg_ha_BLUE > 0, Yield_Mg_ha_BLUE , 0),
         Twt_kg_m3_BLUE = ifelse(Twt_kg_m3_BLUE < (mean.twt + 2.5*iq.twt), Twt_kg_m3_BLUE, NA),
         Twt_kg_m3_BLUE = ifelse(Twt_kg_m3_BLUE > (mean.twt - 2.5*iq.twt), Twt_kg_m3_BLUE, NA),
         Plant_Height_cm_BLUE = ifelse((!(Plant_Height_cm_BLUE > 300 & Ear_Height_cm_BLUE < 100)),Plant_Height_cm_BLUE, NA),
         Plant_Height_cm_BLUE = ifelse((!(Plant_Height_cm_BLUE < 180 & Ear_Height_cm_BLUE > 150)), Plant_Height_cm_BLUE, NA),
         Ear_Height_cm_BLUE = ifelse((!(Plant_Height_cm_BLUE > 300 & Ear_Height_cm_BLUE < 100)),Ear_Height_cm_BLUE, NA),
         Ear_Height_cm_BLUE = ifelse((!(Plant_Height_cm_BLUE < 180 & Ear_Height_cm_BLUE > 150)), Ear_Height_cm_BLUE, NA),
         Grain_Moisture_BLUE = ifelse(Grain_Moisture_BLUE < 35, Grain_Moisture_BLUE, NA),
         Grain_Moisture_BLUE = ifelse(Grain_Moisture_BLUE > 12, Grain_Moisture_BLUE, NA),
         Silk_DAP_BLUE = ifelse(Silk_DAP_BLUE >= 40, Silk_DAP_BLUE, NA),
         Lodging_BLUE = ifelse(Lodging_BLUE < 0, 0, Lodging_BLUE),
         Lodging_BLUE = ifelse(Lodging_BLUE > 1, 1, Lodging_BLUE))

#do final summary check of pred.qc
print("Summary of qc-filtered BLUEs")
summary(pred.qc)

#summarize the weights by environment, some too high
print("Summary of weights for BLUEs in stage 2 analysis by environment")
wt.cols = grep("_weight$", names(pred.qc), value = T)
pred.qc.check = group_by(pred.qc, Env) %>%
  summarise(YDWtMean = mean(Yield_Mg_ha_weight, na.rm = T),
            YDWtMin = min(Yield_Mg_ha_weight, na.rm = T),
            YDWtMax = max(Yield_Mg_ha_weight, na.rm = T),
            EHWtMean = mean(Ear_Height_cm_weight, na.rm = T),
            EHWtMin = min(Ear_Height_cm_weight, na.rm = T),
            EHWtMax = max(Ear_Height_cm_weight, na.rm = T),
            PHWtMean = mean(Plant_Height_cm_weight, na.rm = T),
            PHWtMin = min(Plant_Height_cm_weight, na.rm = T),
            PHWtMax = max(Plant_Height_cm_weight, na.rm = T),
            GMWtMean = mean(Grain_Moisture_weight, na.rm = T),
            GMWtMin = min(Grain_Moisture_weight, na.rm = T),
            GMWtMax = max(Grain_Moisture_weight, na.rm = T),
            LDWtMean = mean(Lodging_weight, na.rm = T),
            LDWtMin = min(Lodging_weight, na.rm = T),
            LDWtMax = max(Lodging_weight, na.rm = T),
            PLWtMean = mean(Pollen_DAP_weight, na.rm = T),
            PLWtMin = min(Pollen_DAP_weight, na.rm = T),
            PLWtMax = max(Pollen_DAP_weight, na.rm = T),
            SLWtMean = mean(Silk_DAP_weight, na.rm = T),
            SLWtMin = min(Silk_DAP_weight, na.rm = T),
            SLWtMax = max(Silk_DAP_weight, na.rm = T),
            TWWtMean = mean(Twt_kg_m3_weight, na.rm = T),
            TWWtMin = min(Twt_kg_m3_weight, na.rm = T),
            TWWtMax = max(Twt_kg_m3_weight, na.rm = T)
            )

summary(pred.qc.check)

#remove 'hybrid' B47/B47 from BLUEs, it's clearly a mistake
pred.qc %>% filter(Hybrid == "B47/B47")#show these three bad records
pred.qc = pred.qc %>% filter(Hybrid != "B47/B47")

#write the qc filtered BLUEs to hard drive
write.csv(pred.qc, file = paste0(out_path, "G2F_2014-16_Stage1_BLUEs_wide.csv"), row.names = F, quote = F)

#remake the pred.df long version by gathering the newly filtered pred.wide
pred.df.qc = gather(pred.qc, key = Trait, value = Value, -Env, -Hybrid) 
pred.df.qc.BLUE = pred.df.qc %>% filter(grepl("BLUE", Trait, fixed = T)) %>% 
  mutate(Trait = sub("_BLUE", "", Trait)) %>%
  rename(BLUE = Value)
pred.df.qc.wt = pred.df.qc %>% filter(grepl("_weight", Trait, fixed = T)) %>% 
  mutate(Trait = sub("_weight", "", Trait)) %>%
  rename(Weight = Value)
pred.df.qc = merge(pred.df.qc.BLUE, pred.df.qc.wt)

write.csv(pred.df.qc, file = paste0(out_path, "G2F_2014-16_Stage1_BLUEs.csv"), row.names = F, quote = F)

write.csv(bad.envs, file = paste0(out_path, "G2F_2014_16_Bad_Env_Trait_Combinations.csv"), row.names = F, quote = F)
